/*
 * gml_Errors.h --
 *
 *    Manages error codes and their corresponding textual explanation.
 */
/*
 *  Copyright (c) 1995-2002 CLIPS-IMAG
 *
 *  See the file "gml_LicenseTerms.txt" for information on usage and redistribution
 *  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 *  Created on July, 1995 (FB).
 */

#ifndef __GMLERRORS__
#define __GMLERRORS__
  #if defined(__cplusplus)
    extern "C" {
  #endif


/**
 * gml_TError --
 *
 * The type representing error codes in Gml.
 */
typedef enum {
  gml_cNoError = 0

} gml_TError;



/*
 * Predefined errors.
 */
extern gml_TError   gml_gErrorMemFull;            /**< no more memory */
extern gml_TError   gml_gErrorWrongParameterVal;  /**< parameter value not acceptable */
extern gml_TError   gml_gDivideByZero;            /**< divide by zero of singularity */
extern gml_TError   gml_gNotInitialized;          /**< attempt to use an uninitialized object */

extern gml_TError   gml_gErrorFileOpenRead;       /**< could not open file for reading */
extern gml_TError   gml_gErrorFileOpenWrite;      /**< could not open file for writing */
extern gml_TError   gml_gErrorFileRead;           /**< while reading data from file */
extern gml_TError   gml_gErrorFileWrite;          /**< while writing data to file */



/**
 * gml_ErrorNew --
 *
 *  Register a new error message with string representation <message> and return its code.
 */
gml_TError          gml_ErrorNew        (char* message);


/**
 * gml_ErrorGetMessage --
 *
 *  Returns the message corresponding to the error code <error>.
 */
char*               gml_ErrorGetMessage (gml_TError error);




/**
 * gml_ErrorDispose --
 *
 *  Releases all error manager resources.
 */
void                gml_ErrorDispose ();


  #if defined(__cplusplus)
    }
  #endif
#endif
