// gml_List.cpp --
//
//    See "gml_List.h".
//
//  Copyright (c) 2003 CLIPS-IMAG
//
//  See the file "gml_LicenseTerms.txt" for information on usage and redistribution
//  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
//
//  Created on May 21, 2003 (FB).

#include "gml_List.h"
#include "gml_Alloc.h"


// gml_TListElem::Init --

gml_TError gml_TListElem::Init ()
{
  fValue        = (void*)NULL;

  this->Reset ();

  return gml_cNoError;
}



// gml_TListElem::Dispose --

void gml_TListElem::Dispose ()
{
  this->RemoveFromList ();
}



// gml_TList::Dispose

void gml_TList::Dispose ()
{
  gml_TListElem*    current = fHead;
  gml_TListElem*    tmp;
  
  while (current != (gml_TListElem*)NULL) {
    tmp     = current;
    current = current->fNext;
    gml_DisposeAndDelete (tmp);
  }

  this->Reset ();
}



// gml_TList::InsertListAfterElem

void gml_TList::InsertListAfterElem (gml_TList* list, gml_TListElem* elem)
{
  gml_TListElem *elemPtr = list->fHead;

  if ((elemPtr == NULL) || ((elem != (gml_TListElem*)NULL) && (elem->fList != this)))
    return;

  while (elemPtr != NULL) {
    elemPtr->fList = this;
    elemPtr = elemPtr->fNext;
  }
  
  if (elem == NULL) {
  
    // Insert at head.

    if (fHead != NULL)
      fHead->fPrevious  = list->fTail;

    list->fTail->fNext  = fHead;
    fHead               = list->fHead;

  } else {

    // Insert inside.

    if (elem->fNext != NULL)
      elem->fNext->fPrevious  = list->fTail;

    list->fTail->fNext        = elem->fNext;
    list->fHead->fPrevious    = elem;
    elem->fNext               = list->fHead;
  }

  if (fTail == elem)
    fTail = list->fTail;

  list->Reset ();
}



// gml_TList::AppendValue

gml_TError gml_TList::AppendValue (void* value, gml_TListElem** newElem)
{
  gml_TError        err;
  gml_TListElem*    elem;
  
  if ((err = gml_NewAndInit (elem)) != gml_cNoError)
    return err;

  this->AppendElem (elem);

  elem->fValue = value;
  if (newElem != (gml_TListElem**)NULL)
    *newElem = elem;
  
  return gml_cNoError;
}



// gml_TList::InsertValue

gml_TError gml_TList::InsertValue (void* value, gml_TListElem** newElem)
{
  gml_TError        err;
  gml_TListElem*    elem;
  
  if ((err = gml_NewAndInit (elem)) != gml_cNoError)
    return err;

  this->InsertElem (elem);
  elem->fValue = value;
  if (newElem != (gml_TListElem**)NULL)
    *newElem = elem;
  
  return gml_cNoError;
}


// AppendValuetAfterElem --

gml_TError gml_TList::AppendValuetAfterElem (void* value, gml_TListElem* elem, gml_TListElem** newElem)
{
  gml_TError        err;
  gml_TListElem*    elemPtr;
  
  if ((err = gml_NewAndInit (elemPtr)) != gml_cNoError)
    return err;
  
  elemPtr->fList = this;
  elemPtr->fValue = value;
  
  if (elem == NULL) {
    // Insert at head.    
    if (fHead != NULL)
      fHead->fPrevious  = elemPtr;
    
    elemPtr->fNext      = fHead; 
    fHead               = elemPtr;    
  } else {    
    // Insert inside.    
    if (elem->fNext != NULL)
      elem->fNext->fPrevious  = elemPtr;
    
    elemPtr->fNext        = elem->fNext;
    elemPtr->fPrevious    = elem;
    elem->fNext           = elemPtr;
  }
  
  if (fTail == elem)
    fTail = elemPtr;
  
  if (newElem != (gml_TListElem**)NULL)
    *newElem = elemPtr;
  
  return err;
}

// gml_TList::ValueElem --

gml_TListElem* gml_TList::ValueElem (void* value)
{
  gml_TListElem*    current = fHead;
  
  while ((current != (gml_TListElem*)NULL) && (current->fValue != value))
    current = current->fNext;

  return current;
}



// gml_TList::RemoveValue

void gml_TList::RemoveValue (void* value)
{
  gml_TListElem* elem = this->ValueElem (value);

  if (elem != (gml_TListElem*)NULL) {
    this->RemoveElem (elem);
    gml_DisposeAndDelete (elem);
  }
}



// gml_TList::IsEquivalentValueMember --

gml_TBoolean gml_TList::IsEquivalentValueMember (void*                value,
                                                 gml_TListSortFunc*   func)
{
  gml_TListElem* elem = fHead;

  while ((elem != (gml_TListElem*)NULL) &&
         ((*func) (elem->fValue, value) != 0))
    elem = elem->fNext;

  return (gml_TBoolean)(elem != (gml_TListElem*)NULL);
}



// gml_ListSortStringFunc --
// gml_ListSortIntFunc --

int gml_ListSortStringFunc (void* left, void* right)
{
  return strcmp ((char*)left, (char*)right);  
}

int gml_ListSortIntFunc (void* left, void* right)
{
  return ((int)left == (int)right) ? 0 : (((int)left > (int)right) ? 1 : -1);
}



// gmlMergeSort --

static void gmlMergeSort (gml_TListElem**     array,
                          int                 size,
                          gml_TListSortFunc*  func,
                          gml_TListElem**     tmp)
{
  gml_TListElem*    swap;
  gml_TListElem**   c1;
  gml_TListElem**   c1stop;
  gml_TListElem**   c2;
  gml_TListElem**   c2stop;
  gml_TListElem**   dest;
  int               mid;
  int               nb;

  // Process base cases.

  if (size < 2)
    return;

  if (size == 2) {
    if ((*func) (array[0]->Value (), array[1]->Value ()) > 0) {
      swap        = array[0];
      array[0]    = array[1];
      array[1]    = swap;
    }
    return;
  }

  // Process general case: split in two, sort each, merge the two.

  mid = size / 2;
  
  c1      = array;
  c1stop  = array + mid;
  c2      = c1stop;
  c2stop  = c2 + (size-mid);

  gmlMergeSort (c1, mid,        func, tmp);
  gmlMergeSort (c2, size - mid, func, tmp);

  dest    = tmp;
  while ((c1 < c1stop) || (c2 < c2stop)) {
    nb = 0;
    while ((c1+nb < c1stop) &&
            ((c2 == c2stop) || ((*func) (c1[nb]->Value (), c2[0]->Value ()) <= 0)))
      nb++;
    if (nb) {
      memcpy (dest, c1, nb * sizeof (*c1));
      dest  += nb;
      c1    += nb;
    }
  
    nb = 0;
    while ((c2+nb < c2stop) && 
            ((c1 == c1stop) || ((*func) (c2[nb]->Value (), c1[0]->Value ()) <= 0)))
      nb++;
    if (nb) {
      memcpy (dest, c2, nb * sizeof (*c2));
      dest  += nb;
      c2    += nb;
    }
  }

  memcpy (array, tmp, size * sizeof (*array));
}



// gml_TList::Sort --
//
//  Sorts the list according to the comparison function <func>.

gml_TError gml_TList::Sort (gml_TListSortFunc* func)
{
  gml_TError        err;
  int               i, j;
  gml_TListElem*    current     = fHead;
  gml_TListElem**   array;

  // Count number of elements in the list, return if 0.

  i = 0;
  while (current != (gml_TListElem*)NULL) {
    i++;
    current = current->Next ();
  }

  if (i == 0)
    return gml_cNoError;

  // Build a contiguous array of pointers to elements.
  // Add a temporary space of the size of the array at
  //  the end, used for merging.

  err = gml_Malloc (array, i+i);
  if (err != gml_cNoError)
    return err;

  j = 0;
  current = fHead;
  while (current != (gml_TListElem*)NULL) {
    array[j++] = current;
    current = current->Next ();
  }

  // Merge-Sort the array. Update the list.

  gmlMergeSort (array, i, func, array + i);

  for (j = 0; j < i-1; j++) {
    array[j]->fNext       = array[j+1];
    array[j+1]->fPrevious = array[j];
  }
  array[0]->fPrevious = (gml_TListElem*)NULL;
  array[i-1]->fNext   = (gml_TListElem*)NULL;
  fHead = array[0];
  fTail = array[i-1];

  gml_Free (array);

  return gml_cNoError;
}




